@@ -1,4 +1,5 @@
 use Cwd;
+use Config;
 
 $cur_dir = cwd();
 $failed = 0;
@@ -7,6 +8,7 @@ $num_tests = 2;
 die "Must be in build directory\n" unless -d 'blib';
 
 $ENV{MINIVEND_ROOT} = "$cur_dir/blib";
+$ENV{MINIVEND_BADPOSIX} = 1;
 
 open(CONFIG, ">$ENV{MINIVEND_ROOT}/minivend.cfg")
 	or die "open: $!\n";
@@ -28,6 +30,12 @@ mkdir ("$ENV{MINIVEND_ROOT}/etc", 0777);
 mkdir ("$ENV{MINIVEND_ROOT}/pages", 0777);
 mkdir ("$ENV{MINIVEND_ROOT}/products", 0777);
 mkdir ("$ENV{MINIVEND_ROOT}/session", 0777);
+if( $ENV{PERL5LIB} ) {
+	$ENV{PERL5LIB} .= ":$cur_dir/extra";
+}
+else {
+	$ENV{PERL5LIB} = "$cur_dir/extra";
+}
 
 open(CONFIG, ">$ENV{MINIVEND_ROOT}/products/products.asc")
 	or die "open: $!\n";
@@ -49,14 +57,14 @@ EOF
 close CONFIG;
 
 print "server.......";
-if ( system "dist/bin/minivend -serve -u >/dev/null" ) {
+if ( system "$Config{'perlpath'} dist/bin/minivend -serve -u >/dev/null" ) {
 	print "not ok 1\n";
 }
 else {
 	print "ok 1\n";
 }
 
-sleep 1;
+sleep 3;
 open(PID, "$ENV{MINIVEND_ROOT}/etc/minivend.pid") or die "read PID file: $!\n";
 $pid = <PID>;
 
@@ -96,8 +104,8 @@ sub send_environment () {
 	return $val;
 }
 
-$SIG{PIPE} = sub { die_page("signal"); };
-$SIG{ALRM} = sub { server_not_running(); exit 1; };
+$SIG{PIPE} = sub { die("signal"); };
+$SIG{ALRM} = sub { die("not communicating with server\n"); exit 1; };
 
 alarm $LINK_TIMEOUT;
 
